#include <iostream>
#include <stdexcept>
#include <boost/shared_ptr.hpp>

// Superclass Person
class Person {
private:
   std::string _name;
   std::string _birthday;

public:
   Person(const std::string& name, const std::string& birthday) :
      _name(name), _birthday(birthday) {
 
   }

   const std::string& getName() const {
      return _name;
   }
};

// class Student, inherits from Person
class Student : public Person {
private:
   int matNum;
public:
   Student(const std::string& name, const std::string& birthday) :
      Person(name, birthday) {
      std::cout << "Student " << getName() << " constructed" << std::endl;
   }   
 
   ~Student() {
      std::cout << "Student " << getName() << " destructed" << std::endl;
   }

   int getMatNum() {
      return matNum;
   }
   
   void setMatNum(int matNum) {
      this->matNum = matNum;
   }
};

/*
 * test function to demonstrate usage of smart-pointers in context of
 * exceptions
 * NOTE: If no exception handling is provided - not even the destructor
 *       of automatic variables is called!
 */
void testFunc() {
   boost::shared_ptr<Student> s = boost::shared_ptr<Student>(new Student("Fritz", ""));
   Student s2("Fritz2", "");
   Student *s3 = new Student("Fritz3", "");
   std::cout << s3->getName() << std::endl;
   std::cout << s->getName() << std::endl;

   throw std::runtime_error("my runtime error");
   /*
   execute many operations
   */
   delete s3;

}

int main() {
   std::cout << "Hello World!" << std::endl;
   
   // Important: if no try-catch is used, not all destructors are called!
   // Give it a try!
   try {
      testFunc();   
   } catch (std::exception ex) {

   }
}
